# Generated by Django 4.2.12 on 2024-05-31 14:07

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailimagecaptions", "0005_auto_20231121_1134"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="captionedimage",
            options={
                "permissions": [("choose_image", "Can choose image")],
                "verbose_name": "image",
                "verbose_name_plural": "images",
            },
        ),
    ]



from django.apps import AppConfig


class WagtailImageCaptionsConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "wagtailimagecaptions"

    def ready(self):
        # Register CaptionedImage for referencing (usage stats).
        from wagtail.models.reference_index import ReferenceIndex

        from .models import CaptionedImage

        ReferenceIndex.register_model(CaptionedImage)

        # Implicitly connect signal handlers decorated with @receiver.
        from . import signals  # noqa



# Generated by Django 4.2 on 2023-07-19 11:10

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("wagtailimagecaptions", "0001_initial"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="ExtendedRendition",
            new_name="CaptionedRendition",
        ),
    ]



# Generated by Django 5.1 on 2024-11-30 14:00

import django.db.models.deletion
import wagtail.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("wagtailcore", "0094_alter_page_locale"),
        ("wagtailimagecaptions", "0006_alter_captionedimage_options"),
    ]

    operations = [
        migrations.CreateModel(
            name="BlogPage",
            fields=[
                (
                    "page_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="wagtailcore.page",
                    ),
                ),
                ("body", wagtail.fields.RichTextField()),
                ("created_on", models.DateTimeField(auto_now_add=True, verbose_name="Created On")),
                (
                    "thumbnail",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to="wagtailimagecaptions.captionedimage",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("wagtailcore.page",),
        ),
    ]



from django.db import models
from wagtail.admin.panels import FieldPanel
from wagtail.fields import RichTextField
from wagtail.images import get_image_model_string
from wagtail.models import Page

IMAGE_MODEL_CLASS = get_image_model_string()


class BlogPage(Page):
    body = RichTextField()
    created_on = models.DateTimeField(verbose_name="Created On", auto_now_add=True)
    thumbnail = models.ForeignKey(
        IMAGE_MODEL_CLASS, null=True, blank=True, on_delete=models.SET_NULL, related_name="+"
    )

    parent_page_types = ["wagtailcore.Page"]
    subpage_types = []

    content_panels = Page.content_panels + [
        FieldPanel("body"),
        FieldPanel("thumbnail"),
    ]



import hashlib
import logging
from os.path import basename

from django.core.files.images import ImageFile
from PIL import Image as PILImage
from PIL.IptcImagePlugin import getiptcinfo
from wagtail.images import get_image_model

logger = logging.getLogger(__name__)


def imagefile_to_model(image_file: ImageFile):
    """
    Converts an ImageFile to our image model.
    """
    ImageModel = get_image_model()

    with image_file.open(mode="rb") as f:
        file_hash = hashlib.sha1(f.read()).hexdigest()

        try:
            image, created = ImageModel.objects.get_or_create(
                file_hash=file_hash,
                defaults={"title": basename(image_file.name), "file": image_file},
            )
            return image
        except ImageModel.MultipleObjectsReturned:
            logger.error(
                "Multiple versions of %s (%s) found. Returning the first one found.",
                basename(image_file.name),
                file_hash,
            )
            return ImageModel.objects.filter(file_hash=file_hash).first()


def parse_iptc(image_file: ImageFile) -> dict:
    """
    Extracts IPTC data from an image (tiff, jpeg). For more inforation see:
        https://www.iptc.org/std/IIM/3.0/specification/IIMV3.PDF
    """
    try:
        image = PILImage.open(image_file)
        iptc = getiptcinfo(image)
    except FileNotFoundError as fnfe:
        logger.warning(fnfe)
        return {}
    except ValueError as ve:
        logger.warning(ve)
        return {}

    iptc_dict = {}

    if not iptc:
        logger.info("Image did not contain IPTC data.")
        return iptc_dict

    def decode(v):
        if isinstance(v, bytes):
            try:
                return bytes.decode(v)
            except UnicodeDecodeError as ude:
                image_file_name = image_file.file.name
                logger.error("Wagtail Image Captions Error for %s: %s", image_file_name, ude)
        elif isinstance(v, list):
            return [decode(item) for item in v]
        elif isinstance(v, str):
            return v

    # fmt: off
    for k, v in iptc.items():
        if k == (2, 5,):
            iptc_dict["object_name"] = decode(v)
        elif k == (2, 7,):
            iptc_dict["edit_status"] = decode(v)
        elif k == (2, 25,):
            iptc_dict["keywords"] = decode(v)
        elif k == (2, 30,):
            iptc_dict["release_date"] = decode(v)
        elif k == (2, 35,):
            iptc_dict["release_time"] = decode(v)
        elif k == (2, 37,):
            iptc_dict["expiration_date"] = decode(v)
        elif k == (2, 38,):
            iptc_dict["expiration_time"] = decode(v)
        elif k == (2, 40,):
            iptc_dict["instructions"] = decode(v)
        elif k == (2, 40,):
            iptc_dict["instructions"] = decode(v)
        elif k == (2, 42,):
            iptc_dict["action_advised"] = decode(v)
        elif k == (2, 80,):
            iptc_dict["byline"] = decode(v)
        elif k == (2, 85,):
            iptc_dict["byline_title"] = decode(v)
        elif k == (2, 90,):
            iptc_dict["city"] = decode(v)
        elif k == (2, 92,):
            iptc_dict["sub_location"] = decode(v)
        elif k == (2, 95,):
            iptc_dict["province_state"] = decode(v)
        elif k == (2, 100,):
            iptc_dict["country"] = decode(v)
        elif k == (2, 105,):
            iptc_dict["headline"] = decode(v)
        elif k == (2, 110,):
            iptc_dict["credit"] = decode(v)
        elif k == (2, 116,):
            iptc_dict["copyright_notice"] = decode(v)
        elif k == (2, 120,):
            iptc_dict["caption"] = decode(v)
        elif k == (2, 122,):
            iptc_dict["writer_editor"] = decode(v)
    # fmt: on

    return {k: v for k, v in iptc_dict.items() if v}



# Generated by Django 5.1 on 2024-11-29 13:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailimagecaptions", "0006_alter_captionedimage_options"),
    ]

    operations = [
        migrations.AddField(
            model_name="captionedimage",
            name="description",
            field=models.CharField(blank=True, default="", max_length=255, verbose_name="description"),
        ),
    ]



import os
import uuid

from django.conf import settings
from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from wagtail.coreutils import string_to_ascii
from wagtail.fields import RichTextField
from wagtail.images.models import AbstractImage, AbstractRendition, Image
from wagtail.search import index


class CaptionedImage(AbstractImage):
    uuid = models.UUIDField(db_index=True, default=uuid.uuid4, editable=False)
    alt = models.CharField(
        max_length=255,
        blank=True,
        help_text=_("Set the image alt text used for accessibility."),
    )
    caption = RichTextField(features=["bold", "italic"], null=True, blank=True, help_text="Set the image caption.")
    credit = models.CharField(
        max_length=255,
        blank=True,
        help_text=_("The name of the provider (e.g. AP, Getty, Reuters)."),
    )
    byline = models.CharField(
        max_length=255,
        blank=True,
        help_text=_("The name(s) of the creator(s) of the image."),
    )
    usage_terms = models.CharField(
        max_length=255,
        blank=True,
        help_text=_(
            "Rights information and usage limitations associated with the image, including any special restrictions or instructions."
        ),
    )
    copyright_notice = models.CharField(
        max_length=255,
        blank=True,
        help_text=_("Any necessary copyright notice(s)."),
    )
    iptc_data = models.JSONField(null=True, blank=True)

    admin_form_fields = Image.admin_form_fields + (
        "credit",
        "byline",
        "alt",
        "caption",
        "usage_terms",
        "copyright_notice",
    )

    search_fields = AbstractImage.search_fields + [
        index.SearchField("uuid"),
        index.SearchField("caption"),
    ]

    class Meta(AbstractImage.Meta):
        verbose_name = _("image")
        verbose_name_plural = _("images")
        permissions = [
            ("choose_image", "Can choose image"),
        ]

    @property
    def default_alt_text(self):
        """Return our stored alt value, otherwise Wagtail defaults to the title."""
        if self.alt:
            return self.alt
        return super().default_alt_text

    def get_upload_to(self, filename):
        """Overrides the `get_upload_to` method to include set date paths."""
        folder_name = "original_images"

        if (
            hasattr(settings, "WAGTIALIMAGECAPTIONS_UPLOAD_TO_DATE_PATH")
            and settings.WAGTIALIMAGECAPTIONS_UPLOAD_TO_DATE_PATH
        ):
            now = timezone.now()
            date_path = now.strftime(settings.WAGTIALIMAGECAPTIONS_UPLOAD_TO_DATE_PATH)
            folder_name = os.path.join(folder_name, date_path)

        # ...now continue with Wagtail source.
        filename = self.file.field.storage.get_valid_name(filename)

        # convert the filename to simple ascii characters and then
        # replace non-ascii characters in filename with _ , to sidestep issues with filesystem encoding
        filename = "".join((i if ord(i) < 128 else "_") for i in string_to_ascii(filename))

        # Truncate filename so it fits in the 100 character limit
        # https://code.djangoproject.com/ticket/9893
        full_path = os.path.join(folder_name, filename)
        if len(full_path) >= 95:
            chars_to_trim = len(full_path) - 94
            prefix, extension = os.path.splitext(filename)
            filename = prefix[:-chars_to_trim] + extension
            full_path = os.path.join(folder_name, filename)

        return full_path


class CaptionedRendition(AbstractRendition):
    image = models.ForeignKey(CaptionedImage, on_delete=models.CASCADE, related_name="renditions")

    class Meta:
        unique_together = (("image", "filter_spec", "focal_point_key"),)

    def get_upload_to(self, filename):
        """Overrides the `get_upload_to` method to include set date paths."""
        if (
            hasattr(settings, "WAGTIALIMAGECAPTIONS_UPLOAD_TO_DATE_PATH")
            and settings.WAGTIALIMAGECAPTIONS_UPLOAD_TO_DATE_PATH
        ):
            now = timezone.now()
            date_path = now.strftime(settings.WAGTIALIMAGECAPTIONS_UPLOAD_TO_DATE_PATH)
            filename = self.file.field.storage.get_valid_name(filename)

            # Setting 'images' here to remain consistent with the Wagtail method.
            return os.path.join("images", date_path, filename)

        return super().get_upload_to(filename)



# Generated by Django 4.2 on 2023-11-21 11:28

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):
    dependencies = [
        ("wagtailimagecaptions", "0002_rename_extendedrendition_captionedrendition"),
    ]

    operations = [
        migrations.AddField(
            model_name="captionedimage",
            name="byline",
            field=models.CharField(blank=True, help_text="The name(s) of the creator(s) of the image.", max_length=255),
        ),
        migrations.AddField(
            model_name="captionedimage",
            name="copyright_notice",
            field=models.CharField(blank=True, help_text="Any necessary copyright notice(s).", max_length=255),
        ),
        migrations.AddField(
            model_name="captionedimage",
            name="usage_terms",
            field=models.CharField(
                blank=True,
                help_text="Rights information and usage limitations associated with the image, including any special restrictions or instructions.",
                max_length=255,
            ),
        ),
        migrations.AddField(
            model_name="captionedimage",
            name="uuid",
            field=models.UUIDField(db_index=True, default=uuid.uuid4, null=True),
        ),
        migrations.AlterField(
            model_name="captionedimage",
            name="credit",
            field=models.CharField(
                blank=True, help_text="The name of the provider (e.g. AP, Getty, Reuters).", max_length=255
            ),
        ),
    ]
